#!/bin/bash

# Default: no --mpi, i_value unset, dry flag off
use_mpi=false
user_i_value=""
user_w_value=""
dry_run=false

# Parse arguments
while [[ $# -gt 0 ]]; do
    case "$1" in
        --mpi)
            use_mpi=true
            shift
            ;;
        --i)
            user_i_value="$2"
            shift 2
            ;;
        --w)
            user_w_value="$2"
            shift 2
            ;;
        --dry)
            dry_run=true
            shift
            ;;
        *)
            echo "Unknown argument: $1"
            echo "Usage: $0 [--mpi] [--i <value>] [--w <value>] [--dry]"
            exit 1
            ;;
    esac
done

# Function to handle actual execution or dry run
run_or_dry() {
    if $dry_run; then
        echo "[DRY RUN] $1"
    else
        echo "Running: $1"  # Print the command that would be executed
        eval "$1"
    fi
}

# ========================
# MPI Benchmark Block
# ========================
if $use_mpi; then
    echo "Running MPI benchmarks..."
    process_counts=(2 4 6 8 10 16)

    for power in {2..9}; do
        n_value=$((10 ** power))
        i_value=${user_i_value:-$((n_value / 10))}
        w_value=${user_w_value:-1}

        echo "-------------------------------------------------------"
        echo "MPI benchmarks for -n $n_value -w $w_value (i=$i_value)"
        echo "-------------------------------------------------------"

        for np in "${process_counts[@]}"; do
            cmd="mpirun -np $np python benchmark.py mpi -n $n_value -i $i_value -w $w_value -k $np"
            run_or_dry "$cmd"
        done
    done

# ========================
# Non-MPI Benchmark Block
# ========================
else
    echo "Running serial, tda, and mp benchmarks..."
    mp_k_values=(1 2 4 6 8 10 16 32)

    for power in {2..9}; do
        n_value=$((10 ** power))
        i_value=${user_i_value:-$((n_value / 10))}
        w_value=${user_w_value:-1}

        echo "-------------------------------------------------------"
        echo "Non-MPI benchmarks for -n $n_value -w $w_value (i=$i_value)"
        echo "-------------------------------------------------------"

        cmd="python benchmark.py serial -n $n_value -i $i_value -w $w_value --k 1"
        run_or_dry "$cmd"

        cmd="python benchmark.py tda -n $n_value -i $i_value -w $w_value -k 1"
        run_or_dry "$cmd"

        for k in "${mp_k_values[@]}"; do
            cmd="python benchmark.py mp -n $n_value -i $i_value -w $w_value -k $k"
            run_or_dry "$cmd"
        done
    done
fi
